/***************************************************************************
 *   Copyright (C) 2003 by Norman Jordan                                   *
 *   njordan@shaw.ca                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <glib.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <glade/glade-xml.h>

#include "gnome-netselect.h"
#include "browser_utils.h"
#include "configuration.h"

#define BROWSER_TYPE_KEY "/apps/gnome-netselect/browser_type"
#define BROWSER_PATH_KEY "/apps/gnome-netselect/browser_path"
#define SEARCH_ENGINE_KEY "/apps/gnome-netselect/search_engine"
#define OPEN_IN_KEY "/apps/gnome-netselect/open_in"
#define OPEN_ACTION_KEY "/apps/gnome-netselect/new_window_action"
#define SEARCH_ACTION_KEY "/apps/gnome-netselect/search_action"
#define GOTO_ACTION_KEY "/apps/gnome-netselect/url_action"
#define NORMAL_IMAGE_PATH_KEY "/apps/gnome-netselect/normal_icon"
#define NEGATIVE_IMAGE_PATH_KEY "/apps/gnome-netselect/negative_icon"

GConfClient *gconf_client = NULL;

gchar *browser_exec_names[] = {"mozilla",
                               "mozilla-firebird",
                               "netscape",
                               "other"};
gint num_browser_types = 4;

void
update_config_string_var(gchar *key, gchar *value)
{
  GConfValue *new_value;

  new_value = gconf_value_new(GCONF_VALUE_STRING);
  gconf_value_set_string(new_value, value);
  gconf_client_set(gconf_client, key, new_value, NULL);
  gconf_value_free(new_value);
}

void
update_config_int_var(gchar *key, gint value)
{
  GConfValue *new_value;

  new_value = gconf_value_new(GCONF_VALUE_INT);
  gconf_value_set_int(new_value, value);
  gconf_client_set(gconf_client, key, new_value, NULL);
  gconf_value_free(new_value);
}

gboolean
initialize_configuration()
{
  gboolean any_browser = FALSE;
  int i = 0;
  gchar *path;
  GConfValue *value;

  gconf_client = gconf_client_get_default();

  if(gconf_client == NULL)
  {
    free_all_globals();
    return FALSE;
  }

  /* Get the browser type */
  value = gconf_client_get(gconf_client,
                           BROWSER_TYPE_KEY,
                           NULL);
  if(value != NULL && value->type != GCONF_VALUE_INVALID &&
    value->type != GCONF_VALUE_STRING)
  {
    gconf_value_free(value);
    free_all_globals();
    return FALSE;
  }
  if(value == NULL || value->type == GCONF_VALUE_INVALID)
  {
    browser_type = 0;
    any_browser = TRUE;
    if(value != NULL)
    {
      gconf_value_free(value);
      value = NULL;
    }
  } else
  {
    browser_type = -1;
    for(i = 0;i < num_browser_types;i++)
    {
      if(strcmp(gconf_value_get_string(value), browser_exec_names[i]) == 0)
      {
        browser_type = i;
        break;
      }
    }
    if(browser_type < 0) browser_type = num_browser_types - 1;
  }
  if(value != NULL)
  {
    gconf_value_free(value);
    value = NULL;
  }

  /* Get the browser path */
  value = gconf_client_get(gconf_client,
                           BROWSER_PATH_KEY,
                           NULL);
  if(value != NULL && value->type != GCONF_VALUE_INVALID &&
    value->type != GCONF_VALUE_STRING)
  {
    gconf_value_free(value);
    free_all_globals();
    return FALSE;
  }
  browser_path = NULL;
  if(value == NULL || value->type == GCONF_VALUE_INVALID)
  {
    /* Search for the browser path */
    if(any_browser == TRUE)
    {
      /* Look for any browser */
      for(i = 0;i < num_browser_types;i++)
      {
        path = g_find_program_in_path(browser_exec_names[i]);
        if(path != NULL)
        {
          browser_type = i;
          update_config_string_var(BROWSER_TYPE_KEY, browser_exec_names[i]);
          browser_path = path;
          update_config_string_var(BROWSER_PATH_KEY, path);
          break;
        }
      }
    } else
    {
      /* Check if the selected browser type is in the path */
      path = g_find_program_in_path(browser_exec_names[browser_type]);
      if(path != NULL)
      {
        browser_path = path;
        update_config_string_var(BROWSER_PATH_KEY, path);
      }
    }
  } else
  {
    browser_path = g_strdup(gconf_value_get_string(value));
  }
  if(value != NULL) gconf_value_free(value);
  value = NULL;

  /* Retrieve the search engine */
  value = gconf_client_get(gconf_client,
                           SEARCH_ENGINE_KEY,
                           NULL);
  if(value != NULL && value->type != GCONF_VALUE_INVALID &&
    value->type != GCONF_VALUE_STRING)
  {
    gconf_value_free(value);
    free_all_globals();
    return FALSE;
  }
  search_engine = -1;
  if(value == NULL || value->type == GCONF_VALUE_INVALID)
  {
    /* Go with Google */
    search_engine = 0;
    update_config_string_var(SEARCH_ENGINE_KEY, query_targets[i].name);
  } else
  {
    /* Make sure that it is valid */
    for(i = 0;i < num_search_engines;i++)
    {
      if(g_ascii_strcasecmp(query_targets[i].name, gconf_value_get_string(value)) == 0)
      {
        search_engine = i;
        break;
      }
    }
    if(search_engine < 0) search_engine = 0;
  }
  if(value != NULL) gconf_value_free(value);
  value = NULL;

  /* Retrieve the open in value */
  value = gconf_client_get(gconf_client,
                           OPEN_IN_KEY,
                           NULL);
  if(value != NULL && value->type != GCONF_VALUE_INVALID &&
    value->type != GCONF_VALUE_INT)
  {
    gconf_value_free(value);
    free_all_globals();
    return FALSE;
  }
  if(value == NULL || value->type == GCONF_VALUE_INVALID)
  {
    /* Use new tabs */
    display_action_type = 2;
    update_config_int_var(OPEN_IN_KEY, 2);
  } else
  {
    display_action_type = gconf_value_get_int(value);
    if(display_action_type < 0 || display_action_type > 2)
    {
      display_action_type = 2;
    }
  }
  if(value != NULL) gconf_value_free(value);
  value = NULL;

  /* Retrieve the new window action */
  value = gconf_client_get(gconf_client,
                           OPEN_ACTION_KEY,
                           NULL);
  if(value != NULL && value->type != GCONF_VALUE_INVALID &&
    value->type != GCONF_VALUE_INT)
  {
    gconf_value_free(value);
    free_all_globals();
    return FALSE;
  }
  if(value == NULL || value->type == GCONF_VALUE_INVALID)
  {
    /* Use double click */
    open_browser_action = 0;
    update_config_int_var(OPEN_ACTION_KEY, 0);
  } else
  {
    open_browser_action = gconf_value_get_int(value);
    if(open_browser_action < 0 || open_browser_action > 2)
      open_browser_action = 0;
  }
  if(value != NULL) gconf_value_free(value);
  value = NULL;

  /* Retrieve the search action */
  value = gconf_client_get(gconf_client,
                           SEARCH_ACTION_KEY,
                           NULL);
  if(value != NULL && value->type != GCONF_VALUE_INVALID &&
    value->type != GCONF_VALUE_INT)
  {
    gconf_value_free(value);
    free_all_globals();
    return FALSE;
  }
  if(value == NULL || value->type == GCONF_VALUE_INVALID)
  {
    /* Use SHIFT + click */
    search_action = 2;
    update_config_int_var(SEARCH_ACTION_KEY, 2);
  } else
  {
    search_action = gconf_value_get_int(value);
    if(search_action < 0 || search_action > 2)
      search_action = 2;
  }
  if(value != NULL) gconf_value_free(value);
  value = NULL;

  /* Retrieve the goto action */
  value = gconf_client_get(gconf_client,
                           GOTO_ACTION_KEY,
                           NULL);
  if(value != NULL && value->type != GCONF_VALUE_INVALID &&
    value->type != GCONF_VALUE_INT)
  {
    gconf_value_free(value);
    free_all_globals();
    return FALSE;
  }
  if(value == NULL || value->type == GCONF_VALUE_INVALID)
  {
    /* Use double click */
    goto_action = 1;
    update_config_int_var(GOTO_ACTION_KEY, 1);
  } else
  {
    goto_action = gconf_value_get_int(value);
    if(goto_action < 0 || goto_action > 2)
      goto_action = 1;
  }
  if(value != NULL) gconf_value_free(value);
  value = NULL;

  /* Retrieve the path for the normal icon */
  value = gconf_client_get(gconf_client,
                           NORMAL_IMAGE_PATH_KEY,
                           NULL);
  if(value != NULL && value->type != GCONF_VALUE_INVALID &&
    value->type != GCONF_VALUE_STRING)
  {
    gconf_value_free(value);
    free_all_globals();
    return FALSE;
  }
  if(value == NULL || value->type == GCONF_VALUE_INVALID)
  {
    /* Use double click */
    path = g_build_filename(PACKAGE, NORMAL_IMAGE, NULL);
    normal_image_path = gnome_pixmap_file(path);
    g_free(path);
    if(normal_image_path == NULL)
    {
      gconf_value_free(value);
      free_all_globals();
      return FALSE;
    }
    update_config_string_var(NORMAL_IMAGE_PATH_KEY, normal_image_path);
  } else
  {
    normal_image_path = g_strdup(gconf_value_get_string(value));
  }
  if(value != NULL) gconf_value_free(value);
  value = NULL;

  /* Retrieve the path for the negative icon */
  value = gconf_client_get(gconf_client,
                           NEGATIVE_IMAGE_PATH_KEY,
                           NULL);
  if(value != NULL && value->type != GCONF_VALUE_INVALID &&
    value->type != GCONF_VALUE_STRING)
  {
    gconf_value_free(value);
    free_all_globals();
    return FALSE;
  }
  if(value == NULL || value->type == GCONF_VALUE_INVALID)
  {
    /* Use double click */
    path = g_build_filename(PACKAGE, NEGATIVE_IMAGE, NULL);
    negative_image_path = gnome_pixmap_file(path);
    g_free(path);
    if(negative_image_path == NULL)
    {
      gconf_value_free(value);
      free_all_globals();
      return FALSE;
    }
    update_config_string_var(NEGATIVE_IMAGE_PATH_KEY, negative_image_path);
  } else
  {
    negative_image_path = g_strdup(gconf_value_get_string(value));
  }
  if(value != NULL) gconf_value_free(value);

  return TRUE;
}

void
web_browser_changed(GtkOptionMenu *menu,
                    gpointer data)
{
  GtkWidget *widget;
  gchar *path;

  browser_type = gtk_option_menu_get_history(menu);
  update_config_string_var(BROWSER_TYPE_KEY, browser_exec_names[browser_type]);

  widget = glade_xml_get_widget(interface_desc, "browser_path");
  path = NULL;
  if(browser_type == 3)
  {
    gnome_file_entry_set_filename(GNOME_FILE_ENTRY(widget), "");
    browser_path = NULL;
  } else
  {
    path = g_find_program_in_path(browser_exec_names[browser_type]);
    if(path == NULL)
    {
      gnome_file_entry_set_filename(GNOME_FILE_ENTRY(widget), "");
      browser_path = NULL;
    } else
    {
      gnome_file_entry_set_filename(GNOME_FILE_ENTRY(widget), path);
      browser_path = path;
    }
  }
  if(path == NULL)
    path = "";
  update_config_string_var(BROWSER_PATH_KEY, path);
}

void
browser_path_changed()
{
  GtkWidget *browse_widget;

  browse_widget = glade_xml_get_widget(interface_desc, "browser_path");
  browser_path = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(browse_widget), FALSE);
  if(browser_path == NULL)
  {
    update_config_string_var(BROWSER_PATH_KEY, "");
    gnome_file_entry_set_filename(GNOME_FILE_ENTRY(browse_widget), "");
  } else
  {
    update_config_string_var(BROWSER_PATH_KEY, browser_path);
  }
}

gboolean
browser_path_text_updated(GtkWidget *entry,
                          GdkEventFocus *event,
                          gpointer data)
{
  browser_path_changed();
  return FALSE;
}

void
browser_path_browse_done(GnomeFileEntry *entry,
                         gpointer data)
{
  browser_path_changed();
}

void
search_engine_changed(GtkOptionMenu *menu,
                      gpointer data)
{
  gint i;
  gchar *new_value = NULL;

  search_engine = gtk_option_menu_get_history(menu);
  for(i = 0;i < num_search_engines;i++)
  {
    if(i == search_engine)
    {
      new_value = query_targets[i].name;
    }
  }
  if(new_value == NULL)
    new_value = "";
  update_config_string_var(SEARCH_ENGINE_KEY, new_value);
}

void
open_in_changed(GtkOptionMenu *menu,
                gpointer data)
{
  display_action_type = gtk_option_menu_get_history(menu);
  update_config_int_var(OPEN_IN_KEY, display_action_type);
}

void
open_browser_changed(GtkOptionMenu *menu,
                     gpointer data)
{
  GtkWidget *widget;
  gint other_value;

  open_browser_action = gtk_option_menu_get_history(menu);
  update_config_int_var(OPEN_ACTION_KEY, open_browser_action);

  widget = glade_xml_get_widget(interface_desc, "open_url");
  other_value = gtk_option_menu_get_history(GTK_OPTION_MENU(widget));
  if(other_value == open_browser_action)
  {
    gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 3);
    goto_action = 3;
    update_config_int_var(GOTO_ACTION_KEY, goto_action);
  }

  widget = glade_xml_get_widget(interface_desc, "search");
  other_value = gtk_option_menu_get_history(GTK_OPTION_MENU(widget));
  if(other_value == open_browser_action)
  {
    gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 3);
    search_action = 3;
    update_config_int_var(SEARCH_ACTION_KEY, search_action);
  }
}

void
open_url_changed(GtkOptionMenu *menu,
                 gpointer data)
{
  GtkWidget *widget;
  gint other_value;

  goto_action = gtk_option_menu_get_history(menu);
  update_config_int_var(GOTO_ACTION_KEY, goto_action);

  widget = glade_xml_get_widget(interface_desc, "open_browser");
  other_value = gtk_option_menu_get_history(GTK_OPTION_MENU(widget));
  if(other_value == goto_action)
  {
    gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 3);
    open_browser_action = 3;
    update_config_int_var(OPEN_ACTION_KEY, open_browser_action);
  }

  widget = glade_xml_get_widget(interface_desc, "search");
  other_value = gtk_option_menu_get_history(GTK_OPTION_MENU(widget));
  if(other_value == goto_action)
  {
    gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 3);
    search_action = 3;
    update_config_int_var(SEARCH_ACTION_KEY, search_action);
  }
}

void
search_changed(GtkOptionMenu *menu,
               gpointer data)
{
  GtkWidget *widget;
  gint other_value;

  search_action = gtk_option_menu_get_history(menu);
  update_config_int_var(SEARCH_ACTION_KEY, search_action);

  widget = glade_xml_get_widget(interface_desc, "open_browser");
  other_value = gtk_option_menu_get_history(GTK_OPTION_MENU(widget));
  if(other_value == search_action)
  {
    gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 3);
    open_browser_action = 3;
    update_config_int_var(OPEN_ACTION_KEY, open_browser_action);
  }

  widget = glade_xml_get_widget(interface_desc, "open_url");
  other_value = gtk_option_menu_get_history(GTK_OPTION_MENU(widget));
  if(other_value == search_action)
  {
    gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 3);
    goto_action = 3;
    update_config_int_var(GOTO_ACTION_KEY, goto_action);
  }
}

void
normal_icon_path_changed()
{
  GtkWidget *browse_widget;
  GdkPixbuf *new_pixbuf, *temp_pixbuf;
  gint size;

  browse_widget = glade_xml_get_widget(interface_desc, "normal_icon_path");
  if(normal_image_path != NULL)
    g_free(normal_image_path);
  normal_image_path = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(browse_widget), TRUE);
  if(normal_image_path == NULL)
  {
    update_config_string_var(NORMAL_IMAGE_PATH_KEY, "");
    gnome_file_entry_set_filename(GNOME_FILE_ENTRY(browse_widget), "");
  } else
  {
    new_pixbuf = gdk_pixbuf_new_from_file(normal_image_path, NULL);
    if(new_pixbuf != NULL)
    {
      size = get_applet_size();
      temp_pixbuf = normal_image;
      normal_image = gdk_pixbuf_scale_simple(new_pixbuf, size, size, GDK_INTERP_BILINEAR);
      g_object_unref(new_pixbuf);
      gtk_image_set_from_pixbuf(GTK_IMAGE(image), normal_image);
      g_object_unref(temp_pixbuf);
    }
    update_config_string_var(NORMAL_IMAGE_PATH_KEY, normal_image_path);
  }
}

gboolean
normal_icon_path_text_updated(GtkWidget *entry,
                              GdkEventFocus *event,
                              gpointer data)
{
  normal_icon_path_changed();
  return FALSE;
}

void
normal_icon_path_browse_done(GnomeFileEntry *entry,
                             gpointer data)
{
  normal_icon_path_changed();
}

void
negative_icon_path_changed()
{
  GtkWidget *browse_widget;
  GdkPixbuf *new_pixbuf, *temp_pixbuf;
  gint size;

  browse_widget = glade_xml_get_widget(interface_desc, "negative_icon_path");
  if(negative_image_path != NULL)
    g_free(negative_image_path);
  negative_image_path = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(browse_widget), TRUE);
  if(negative_image_path == NULL)
  {
    update_config_string_var(NEGATIVE_IMAGE_PATH_KEY, "");
    gnome_file_entry_set_filename(GNOME_FILE_ENTRY(browse_widget), "");
  } else
  {
    new_pixbuf = gdk_pixbuf_new_from_file(negative_image_path, NULL);
    if(new_pixbuf != NULL)
    {
      size = get_applet_size();
      temp_pixbuf = negative_image;
      negative_image = gdk_pixbuf_scale_simple(new_pixbuf, size, size, GDK_INTERP_BILINEAR);
      g_object_unref(new_pixbuf);
      g_object_unref(temp_pixbuf);
    }
    update_config_string_var(NEGATIVE_IMAGE_PATH_KEY, negative_image_path);
  }
}

gboolean
negative_icon_path_text_updated(GtkWidget *entry,
                          GdkEventFocus *event,
                          gpointer data)
{
  negative_icon_path_changed();
  return FALSE;
}

void
negative_icon_path_browse_done(GnomeFileEntry *entry,
                         gpointer data)
{
  negative_icon_path_changed();
}

void
update_changed_settings()
{
  GtkWidget *widget;
  gchar *tempString;

  widget = glade_xml_get_widget(interface_desc, "browser_path");
  tempString = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(widget), TRUE);
  if(tempString != NULL && strcmp(tempString, browser_path) != 0)
  {
    update_config_string_var(BROWSER_PATH_KEY, tempString);
    if(browser_path != NULL)
      g_free(browser_path);
    browser_path = tempString;
  } else if(tempString != NULL)
  {
    g_free(tempString);
  }

  widget = glade_xml_get_widget(interface_desc, "normal_icon_path");
  tempString = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(widget), TRUE);
  if(tempString != NULL && strcmp(tempString, normal_image_path) != 0)
  {
    g_free(tempString);
    normal_icon_path_changed();
  } else if(tempString != NULL)
  {
    g_free(tempString);
  }

  widget = glade_xml_get_widget(interface_desc, "negative_icon_path");
  tempString = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(widget), TRUE);
  if(tempString != NULL && strcmp(tempString, negative_image_path) != 0)
  {
    g_free(tempString);
    negative_icon_path_changed();
  } else if(tempString != NULL)
  {
    g_free(tempString);
  }
}
